//------------------------------------------------------------------------------
// SyntaxClone.cpp
// All generated syntax node clone functionality
//
// SPDX-FileCopyrightText: Michael Popoloski
// SPDX-License-Identifier: MIT
//------------------------------------------------------------------------------
#include "slang/syntax/AllSyntax.h"

// This file contains all syntax generated clone implementations.
// It is auto-generated by the syntax_gen.py script under the scripts/ directory.

namespace slang::syntax::shallow {

template<typename T>
SyntaxNode* clone(const T& node, BumpAllocator& alloc) {
    return alloc.emplace<T>(node);
}

}

namespace slang::syntax::deep {

template<typename T>
SyntaxNode* clone(const T& node, BumpAllocator& alloc) {
    return alloc.emplace<T>(node);
}

SyntaxNode* clone(const SyntaxListBase&, BumpAllocator&) {
    return nullptr;
}
SyntaxNode* clone(const AcceptOnPropertyExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<AcceptOnPropertyExprSyntax>(
        node.keyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.condition, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone<PropertyExprSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const ActionBlockSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ActionBlockSyntax>(
        node.statement ? deepClone(*node.statement, alloc) : nullptr,
        node.elseClause ? deepClone(*node.elseClause, alloc) : nullptr
    );
}

SyntaxNode* clone(const AnonymousProgramSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<AnonymousProgramSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.semi.deepClone(alloc),
        *deepClone(node.members, alloc),
        node.endkeyword.deepClone(alloc)
    );
}

SyntaxNode* clone(const AnsiPortListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<AnsiPortListSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone(node.ports, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const AnsiUdpPortListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<AnsiUdpPortListSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone(node.ports, alloc),
        node.closeParen.deepClone(alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ArgumentListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ArgumentListSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone(node.parameters, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ArrayOrRandomizeMethodExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ArrayOrRandomizeMethodExpressionSyntax>(
        *deepClone<ExpressionSyntax>(*node.method, alloc),
        node.with.deepClone(alloc),
        node.args ? deepClone(*node.args, alloc) : nullptr,
        node.constraints ? deepClone(*node.constraints, alloc) : nullptr
    );
}

SyntaxNode* clone(const AssertionItemPortListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<AssertionItemPortListSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone(node.ports, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const AssertionItemPortSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<AssertionItemPortSyntax>(
        *deepClone(node.attributes, alloc),
        node.local.deepClone(alloc),
        node.direction.deepClone(alloc),
        *deepClone<DataTypeSyntax>(*node.type, alloc),
        node.name.deepClone(alloc),
        *deepClone(node.dimensions, alloc),
        node.defaultValue ? deepClone(*node.defaultValue, alloc) : nullptr
    );
}

SyntaxNode* clone(const AssignmentPatternExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<AssignmentPatternExpressionSyntax>(
        node.type ? deepClone(*node.type, alloc) : nullptr,
        *deepClone<AssignmentPatternSyntax>(*node.pattern, alloc)
    );
}

SyntaxNode* clone(const AssignmentPatternItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<AssignmentPatternItemSyntax>(
        *deepClone<ExpressionSyntax>(*node.key, alloc),
        node.colon.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const AttributeInstanceSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<AttributeInstanceSyntax>(
        node.openParen.deepClone(alloc),
        node.openStar.deepClone(alloc),
        *deepClone(node.specs, alloc),
        node.closeStar.deepClone(alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const AttributeSpecSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<AttributeSpecSyntax>(
        node.name.deepClone(alloc),
        node.value ? deepClone(*node.value, alloc) : nullptr
    );
}

SyntaxNode* clone(const BadExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BadExpressionSyntax>(
        *deepClone<ExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const BeginKeywordsDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BeginKeywordsDirectiveSyntax>(
        node.directive.deepClone(alloc),
        node.versionSpecifier.deepClone(alloc)
    );
}

SyntaxNode* clone(const BinSelectWithFilterExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BinSelectWithFilterExprSyntax>(
        *deepClone<BinsSelectExpressionSyntax>(*node.expr, alloc),
        node.with.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.filter, alloc),
        node.closeParen.deepClone(alloc),
        node.matchesClause ? deepClone(*node.matchesClause, alloc) : nullptr
    );
}

SyntaxNode* clone(const BinaryBinsSelectExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BinaryBinsSelectExprSyntax>(
        *deepClone<BinsSelectExpressionSyntax>(*node.left, alloc),
        node.op.deepClone(alloc),
        *deepClone<BinsSelectExpressionSyntax>(*node.right, alloc)
    );
}

SyntaxNode* clone(const BinaryBlockEventExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BinaryBlockEventExpressionSyntax>(
        *deepClone<BlockEventExpressionSyntax>(*node.left, alloc),
        node.orKeyword.deepClone(alloc),
        *deepClone<BlockEventExpressionSyntax>(*node.right, alloc)
    );
}

SyntaxNode* clone(const BinaryConditionalDirectiveExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BinaryConditionalDirectiveExpressionSyntax>(
        *deepClone<ConditionalDirectiveExpressionSyntax>(*node.left, alloc),
        node.op.deepClone(alloc),
        *deepClone<ConditionalDirectiveExpressionSyntax>(*node.right, alloc)
    );
}

SyntaxNode* clone(const BinaryEventExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BinaryEventExpressionSyntax>(
        *deepClone<EventExpressionSyntax>(*node.left, alloc),
        node.operatorToken.deepClone(alloc),
        *deepClone<EventExpressionSyntax>(*node.right, alloc)
    );
}

SyntaxNode* clone(const BinaryExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BinaryExpressionSyntax>(
        node.kind,
        *deepClone<ExpressionSyntax>(*node.left, alloc),
        node.operatorToken.deepClone(alloc),
        *deepClone(node.attributes, alloc),
        *deepClone<ExpressionSyntax>(*node.right, alloc)
    );
}

SyntaxNode* clone(const BinaryPropertyExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BinaryPropertyExprSyntax>(
        node.kind,
        *deepClone<PropertyExprSyntax>(*node.left, alloc),
        node.op.deepClone(alloc),
        *deepClone<PropertyExprSyntax>(*node.right, alloc)
    );
}

SyntaxNode* clone(const BinarySequenceExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BinarySequenceExprSyntax>(
        node.kind,
        *deepClone<SequenceExprSyntax>(*node.left, alloc),
        node.op.deepClone(alloc),
        *deepClone<SequenceExprSyntax>(*node.right, alloc)
    );
}

SyntaxNode* clone(const BindDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BindDirectiveSyntax>(
        *deepClone(node.attributes, alloc),
        node.bind.deepClone(alloc),
        *deepClone<NameSyntax>(*node.target, alloc),
        node.targetInstances ? deepClone(*node.targetInstances, alloc) : nullptr,
        *deepClone<MemberSyntax>(*node.instantiation, alloc)
    );
}

SyntaxNode* clone(const BindTargetListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BindTargetListSyntax>(
        node.colon.deepClone(alloc),
        *deepClone(node.targets, alloc)
    );
}

SyntaxNode* clone(const BinsSelectConditionExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BinsSelectConditionExprSyntax>(
        node.binsof.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<NameSyntax>(*node.name, alloc),
        node.closeParen.deepClone(alloc),
        node.intersects ? deepClone(*node.intersects, alloc) : nullptr
    );
}

SyntaxNode* clone(const BinsSelectionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BinsSelectionSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.name.deepClone(alloc),
        node.equals.deepClone(alloc),
        *deepClone<BinsSelectExpressionSyntax>(*node.expr, alloc),
        node.iff ? deepClone(*node.iff, alloc) : nullptr,
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const BitSelectSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BitSelectSyntax>(
        *deepClone<ExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const BlockCoverageEventSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BlockCoverageEventSyntax>(
        node.atat.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<BlockEventExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const BlockStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<BlockStatementSyntax>(
        node.kind,
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.begin.deepClone(alloc),
        node.blockName ? deepClone(*node.blockName, alloc) : nullptr,
        *deepClone(node.items, alloc),
        node.end.deepClone(alloc),
        node.endBlockName ? deepClone(*node.endBlockName, alloc) : nullptr
    );
}

SyntaxNode* clone(const CaseGenerateSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CaseGenerateSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.condition, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.endCase.deepClone(alloc)
    );
}

SyntaxNode* clone(const CasePropertyExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CasePropertyExprSyntax>(
        node.caseKeyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.endcase.deepClone(alloc)
    );
}

SyntaxNode* clone(const CaseStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CaseStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.uniqueOrPriority.deepClone(alloc),
        node.caseKeyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc),
        node.matchesOrInside.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.endcase.deepClone(alloc)
    );
}

SyntaxNode* clone(const CastExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CastExpressionSyntax>(
        *deepClone<ExpressionSyntax>(*node.left, alloc),
        node.apostrophe.deepClone(alloc),
        *deepClone<ParenthesizedExpressionSyntax>(*node.right, alloc)
    );
}

SyntaxNode* clone(const CellConfigRuleSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CellConfigRuleSyntax>(
        node.cell.deepClone(alloc),
        *deepClone<ConfigCellIdentifierSyntax>(*node.name, alloc),
        *deepClone<ConfigRuleClauseSyntax>(*node.ruleClause, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ChargeStrengthSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ChargeStrengthSyntax>(
        node.openParen.deepClone(alloc),
        node.strength.deepClone(alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const CheckerDataDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CheckerDataDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.rand.deepClone(alloc),
        *deepClone<DataDeclarationSyntax>(*node.data, alloc)
    );
}

SyntaxNode* clone(const CheckerDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CheckerDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.name.deepClone(alloc),
        node.portList ? deepClone(*node.portList, alloc) : nullptr,
        node.semi.deepClone(alloc),
        *deepClone(node.members, alloc),
        node.end.deepClone(alloc),
        node.endBlockName ? deepClone(*node.endBlockName, alloc) : nullptr
    );
}

SyntaxNode* clone(const CheckerInstanceStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CheckerInstanceStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        *deepClone<CheckerInstantiationSyntax>(*node.instance, alloc)
    );
}

SyntaxNode* clone(const CheckerInstantiationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CheckerInstantiationSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone<NameSyntax>(*node.type, alloc),
        node.parameters ? deepClone(*node.parameters, alloc) : nullptr,
        *deepClone(node.instances, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ClassDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ClassDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.virtualOrInterface.deepClone(alloc),
        node.classKeyword.deepClone(alloc),
        node.finalSpecifier ? deepClone(*node.finalSpecifier, alloc) : nullptr,
        node.name.deepClone(alloc),
        node.parameters ? deepClone(*node.parameters, alloc) : nullptr,
        node.extendsClause ? deepClone(*node.extendsClause, alloc) : nullptr,
        node.implementsClause ? deepClone(*node.implementsClause, alloc) : nullptr,
        node.semi.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.endClass.deepClone(alloc),
        node.endBlockName ? deepClone(*node.endBlockName, alloc) : nullptr
    );
}

SyntaxNode* clone(const ClassMethodDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ClassMethodDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone(node.qualifiers, alloc),
        *deepClone<FunctionDeclarationSyntax>(*node.declaration, alloc)
    );
}

SyntaxNode* clone(const ClassMethodPrototypeSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ClassMethodPrototypeSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone(node.qualifiers, alloc),
        *deepClone<FunctionPrototypeSyntax>(*node.prototype, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ClassNameSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ClassNameSyntax>(
        node.identifier.deepClone(alloc),
        *deepClone<ParameterValueAssignmentSyntax>(*node.parameters, alloc)
    );
}

SyntaxNode* clone(const ClassPropertyDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ClassPropertyDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone(node.qualifiers, alloc),
        *deepClone<MemberSyntax>(*node.declaration, alloc)
    );
}

SyntaxNode* clone(const ClassSpecifierSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ClassSpecifierSyntax>(
        node.colon.deepClone(alloc),
        node.keyword.deepClone(alloc)
    );
}

SyntaxNode* clone(const ClockingDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ClockingDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.globalOrDefault.deepClone(alloc),
        node.clocking.deepClone(alloc),
        node.blockName.deepClone(alloc),
        node.at.deepClone(alloc),
        *deepClone<EventExpressionSyntax>(*node.event, alloc),
        node.semi.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.endClocking.deepClone(alloc),
        node.endBlockName ? deepClone(*node.endBlockName, alloc) : nullptr
    );
}

SyntaxNode* clone(const ClockingDirectionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ClockingDirectionSyntax>(
        node.input.deepClone(alloc),
        node.inputSkew ? deepClone(*node.inputSkew, alloc) : nullptr,
        node.output.deepClone(alloc),
        node.outputSkew ? deepClone(*node.outputSkew, alloc) : nullptr
    );
}

SyntaxNode* clone(const ClockingItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ClockingItemSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone<ClockingDirectionSyntax>(*node.direction, alloc),
        *deepClone(node.decls, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ClockingPropertyExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ClockingPropertyExprSyntax>(
        *deepClone<TimingControlSyntax>(*node.event, alloc),
        node.expr ? deepClone(*node.expr, alloc) : nullptr
    );
}

SyntaxNode* clone(const ClockingSequenceExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ClockingSequenceExprSyntax>(
        *deepClone<TimingControlSyntax>(*node.event, alloc),
        *deepClone<SequenceExprSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const ClockingSkewSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ClockingSkewSyntax>(
        node.edge.deepClone(alloc),
        node.delay ? deepClone(*node.delay, alloc) : nullptr
    );
}

SyntaxNode* clone(const ColonExpressionClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ColonExpressionClauseSyntax>(
        node.colon.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const CompilationUnitSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CompilationUnitSyntax>(
        *deepClone(node.members, alloc),
        node.endOfFile.deepClone(alloc)
    );
}

SyntaxNode* clone(const ConcatenationExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConcatenationExpressionSyntax>(
        node.openBrace.deepClone(alloc),
        *deepClone(node.expressions, alloc),
        node.closeBrace.deepClone(alloc)
    );
}

SyntaxNode* clone(const ConcurrentAssertionMemberSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConcurrentAssertionMemberSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone<ConcurrentAssertionStatementSyntax>(*node.statement, alloc)
    );
}

SyntaxNode* clone(const ConcurrentAssertionStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConcurrentAssertionStatementSyntax>(
        node.kind,
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.propertyOrSequence.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<PropertySpecSyntax>(*node.propertySpec, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone<ActionBlockSyntax>(*node.action, alloc)
    );
}

SyntaxNode* clone(const ConditionalBranchDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConditionalBranchDirectiveSyntax>(
        node.kind,
        node.directive.deepClone(alloc),
        *deepClone<ConditionalDirectiveExpressionSyntax>(*node.expr, alloc),
        *deepClone(node.disabledTokens, alloc)
    );
}

SyntaxNode* clone(const ConditionalConstraintSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConditionalConstraintSyntax>(
        node.ifKeyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.condition, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone<ConstraintItemSyntax>(*node.constraints, alloc),
        node.elseClause ? deepClone(*node.elseClause, alloc) : nullptr
    );
}

SyntaxNode* clone(const ConditionalExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConditionalExpressionSyntax>(
        *deepClone<ConditionalPredicateSyntax>(*node.predicate, alloc),
        node.question.deepClone(alloc),
        *deepClone(node.attributes, alloc),
        *deepClone<ExpressionSyntax>(*node.left, alloc),
        node.colon.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.right, alloc)
    );
}

SyntaxNode* clone(const ConditionalPathDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConditionalPathDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.predicate, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone<PathDeclarationSyntax>(*node.path, alloc)
    );
}

SyntaxNode* clone(const ConditionalPatternSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConditionalPatternSyntax>(
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.matchesClause ? deepClone(*node.matchesClause, alloc) : nullptr
    );
}

SyntaxNode* clone(const ConditionalPredicateSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConditionalPredicateSyntax>(
        *deepClone(node.conditions, alloc)
    );
}

SyntaxNode* clone(const ConditionalPropertyExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConditionalPropertyExprSyntax>(
        node.ifKeyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.condition, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone<PropertyExprSyntax>(*node.expr, alloc),
        node.elseClause ? deepClone(*node.elseClause, alloc) : nullptr
    );
}

SyntaxNode* clone(const ConditionalStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConditionalStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.uniqueOrPriority.deepClone(alloc),
        node.ifKeyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ConditionalPredicateSyntax>(*node.predicate, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone<StatementSyntax>(*node.statement, alloc),
        node.elseClause ? deepClone(*node.elseClause, alloc) : nullptr
    );
}

SyntaxNode* clone(const ConfigCellIdentifierSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConfigCellIdentifierSyntax>(
        node.library.deepClone(alloc),
        node.dot.deepClone(alloc),
        node.cell.deepClone(alloc)
    );
}

SyntaxNode* clone(const ConfigDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConfigDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.config.deepClone(alloc),
        node.name.deepClone(alloc),
        node.semi1.deepClone(alloc),
        *deepClone(node.localparams, alloc),
        node.design.deepClone(alloc),
        *deepClone(node.topCells, alloc),
        node.semi2.deepClone(alloc),
        *deepClone(node.rules, alloc),
        node.endconfig.deepClone(alloc),
        node.blockName ? deepClone(*node.blockName, alloc) : nullptr
    );
}

SyntaxNode* clone(const ConfigInstanceIdentifierSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConfigInstanceIdentifierSyntax>(
        node.dot.deepClone(alloc),
        node.name.deepClone(alloc)
    );
}

SyntaxNode* clone(const ConfigLiblistSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConfigLiblistSyntax>(
        node.liblist.deepClone(alloc),
        *deepClone(node.libraries, alloc)
    );
}

SyntaxNode* clone(const ConfigUseClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConfigUseClauseSyntax>(
        node.use.deepClone(alloc),
        node.name ? deepClone(*node.name, alloc) : nullptr,
        node.paramAssignments ? deepClone(*node.paramAssignments, alloc) : nullptr,
        node.colon.deepClone(alloc),
        node.config.deepClone(alloc)
    );
}

SyntaxNode* clone(const ConstraintBlockSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConstraintBlockSyntax>(
        node.openBrace.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.closeBrace.deepClone(alloc)
    );
}

SyntaxNode* clone(const ConstraintDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConstraintDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone(node.qualifiers, alloc),
        node.keyword.deepClone(alloc),
        *deepClone(node.specifiers, alloc),
        *deepClone<NameSyntax>(*node.name, alloc),
        *deepClone<ConstraintBlockSyntax>(*node.block, alloc)
    );
}

SyntaxNode* clone(const ConstraintPrototypeSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ConstraintPrototypeSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone(node.qualifiers, alloc),
        node.keyword.deepClone(alloc),
        *deepClone(node.specifiers, alloc),
        *deepClone<NameSyntax>(*node.name, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ContinuousAssignSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ContinuousAssignSyntax>(
        *deepClone(node.attributes, alloc),
        node.assign.deepClone(alloc),
        node.strength ? deepClone(*node.strength, alloc) : nullptr,
        node.delay ? deepClone(*node.delay, alloc) : nullptr,
        *deepClone(node.assignments, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const CopyClassExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CopyClassExpressionSyntax>(
        *deepClone<NameSyntax>(*node.scopedNew, alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const CoverCrossSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CoverCrossSyntax>(
        *deepClone(node.attributes, alloc),
        node.label ? deepClone(*node.label, alloc) : nullptr,
        node.cross.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.iff ? deepClone(*node.iff, alloc) : nullptr,
        node.openBrace.deepClone(alloc),
        *deepClone(node.members, alloc),
        node.closeBrace.deepClone(alloc),
        node.emptySemi.deepClone(alloc)
    );
}

SyntaxNode* clone(const CoverageBinsArraySizeSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CoverageBinsArraySizeSyntax>(
        node.openBracket.deepClone(alloc),
        node.expr ? deepClone(*node.expr, alloc) : nullptr,
        node.closeBracket.deepClone(alloc)
    );
}

SyntaxNode* clone(const CoverageBinsSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CoverageBinsSyntax>(
        *deepClone(node.attributes, alloc),
        node.wildcard.deepClone(alloc),
        node.keyword.deepClone(alloc),
        node.name.deepClone(alloc),
        node.size ? deepClone(*node.size, alloc) : nullptr,
        node.equals.deepClone(alloc),
        *deepClone<CoverageBinInitializerSyntax>(*node.initializer, alloc),
        node.iff ? deepClone(*node.iff, alloc) : nullptr,
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const CoverageIffClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CoverageIffClauseSyntax>(
        node.iff.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const CoverageOptionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CoverageOptionSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const CovergroupDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CovergroupDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.covergroup.deepClone(alloc),
        node.extends.deepClone(alloc),
        node.name.deepClone(alloc),
        node.portList ? deepClone(*node.portList, alloc) : nullptr,
        node.event ? deepClone(*node.event, alloc) : nullptr,
        node.semi.deepClone(alloc),
        *deepClone(node.members, alloc),
        node.endgroup.deepClone(alloc),
        node.endBlockName ? deepClone(*node.endBlockName, alloc) : nullptr
    );
}

SyntaxNode* clone(const CoverpointSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<CoverpointSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone<DataTypeSyntax>(*node.type, alloc),
        node.label ? deepClone(*node.label, alloc) : nullptr,
        node.coverpoint.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.iff ? deepClone(*node.iff, alloc) : nullptr,
        node.openBrace.deepClone(alloc),
        *deepClone(node.members, alloc),
        node.closeBrace.deepClone(alloc),
        node.emptySemi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DPIExportSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DPIExportSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.specString.deepClone(alloc),
        node.c_identifier.deepClone(alloc),
        node.equals.deepClone(alloc),
        node.functionOrTask.deepClone(alloc),
        node.name.deepClone(alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DPIImportSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DPIImportSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.specString.deepClone(alloc),
        node.property.deepClone(alloc),
        node.c_identifier.deepClone(alloc),
        node.equals.deepClone(alloc),
        *deepClone<FunctionPrototypeSyntax>(*node.method, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DataDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DataDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone(node.modifiers, alloc),
        *deepClone<DataTypeSyntax>(*node.type, alloc),
        *deepClone(node.declarators, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DeclaratorSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DeclaratorSyntax>(
        node.name.deepClone(alloc),
        *deepClone(node.dimensions, alloc),
        node.initializer ? deepClone(*node.initializer, alloc) : nullptr
    );
}

SyntaxNode* clone(const DefParamAssignmentSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefParamAssignmentSyntax>(
        *deepClone<NameSyntax>(*node.name, alloc),
        *deepClone<EqualsValueClauseSyntax>(*node.setter, alloc)
    );
}

SyntaxNode* clone(const DefParamSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefParamSyntax>(
        *deepClone(node.attributes, alloc),
        node.defparam.deepClone(alloc),
        *deepClone(node.assignments, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DefaultCaseItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefaultCaseItemSyntax>(
        node.defaultKeyword.deepClone(alloc),
        node.colon.deepClone(alloc),
        *deepClone<SyntaxNode>(*node.clause, alloc)
    );
}

SyntaxNode* clone(const DefaultClockingReferenceSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefaultClockingReferenceSyntax>(
        *deepClone(node.attributes, alloc),
        node.defaultKeyword.deepClone(alloc),
        node.clocking.deepClone(alloc),
        node.name.deepClone(alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DefaultConfigRuleSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefaultConfigRuleSyntax>(
        node.defaultKeyword.deepClone(alloc),
        *deepClone<ConfigLiblistSyntax>(*node.liblist, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DefaultCoverageBinInitializerSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefaultCoverageBinInitializerSyntax>(
        node.defaultKeyword.deepClone(alloc),
        node.sequenceKeyword.deepClone(alloc)
    );
}

SyntaxNode* clone(const DefaultDecayTimeDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefaultDecayTimeDirectiveSyntax>(
        node.directive.deepClone(alloc),
        node.time.deepClone(alloc)
    );
}

SyntaxNode* clone(const DefaultDisableDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefaultDisableDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.defaultKeyword.deepClone(alloc),
        node.disableKeyword.deepClone(alloc),
        node.iffKeyword.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DefaultDistItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefaultDistItemSyntax>(
        node.defaultKeyword.deepClone(alloc),
        node.weight ? deepClone(*node.weight, alloc) : nullptr
    );
}

SyntaxNode* clone(const DefaultExtendsClauseArgSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefaultExtendsClauseArgSyntax>(
        node.openParen.deepClone(alloc),
        node.defaultKeyword.deepClone(alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const DefaultFunctionPortSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefaultFunctionPortSyntax>(
        node.keyword.deepClone(alloc)
    );
}

SyntaxNode* clone(const DefaultNetTypeDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefaultNetTypeDirectiveSyntax>(
        node.directive.deepClone(alloc),
        node.netType.deepClone(alloc)
    );
}

SyntaxNode* clone(const DefaultPropertyCaseItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefaultPropertyCaseItemSyntax>(
        node.defaultKeyword.deepClone(alloc),
        node.colon.deepClone(alloc),
        *deepClone<PropertyExprSyntax>(*node.expr, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DefaultRsCaseItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefaultRsCaseItemSyntax>(
        node.defaultKeyword.deepClone(alloc),
        node.colon.deepClone(alloc),
        *deepClone<RsProdItemSyntax>(*node.item, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DefaultSkewItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefaultSkewItemSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone<ClockingDirectionSyntax>(*node.direction, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DefaultTriregStrengthDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefaultTriregStrengthDirectiveSyntax>(
        node.directive.deepClone(alloc),
        node.strength.deepClone(alloc)
    );
}

SyntaxNode* clone(const DeferredAssertionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DeferredAssertionSyntax>(
        node.hash.deepClone(alloc),
        node.zero.deepClone(alloc),
        node.finalKeyword.deepClone(alloc)
    );
}

SyntaxNode* clone(const DefineDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DefineDirectiveSyntax>(
        node.directive.deepClone(alloc),
        node.name.deepClone(alloc),
        node.formalArguments ? deepClone(*node.formalArguments, alloc) : nullptr,
        *deepClone(node.body, alloc)
    );
}

SyntaxNode* clone(const Delay3Syntax& node, BumpAllocator& alloc) {
    return alloc.emplace<Delay3Syntax>(
        node.hash.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.delay1, alloc),
        node.comma1.deepClone(alloc),
        node.delay2 ? deepClone(*node.delay2, alloc) : nullptr,
        node.comma2.deepClone(alloc),
        node.delay3 ? deepClone(*node.delay3, alloc) : nullptr,
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const DelaySyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DelaySyntax>(
        node.kind,
        node.hash.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.delayValue, alloc)
    );
}

SyntaxNode* clone(const DelayedSequenceElementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DelayedSequenceElementSyntax>(
        node.doubleHash.deepClone(alloc),
        node.delayVal ? deepClone(*node.delayVal, alloc) : nullptr,
        node.openBracket.deepClone(alloc),
        node.op.deepClone(alloc),
        node.range ? deepClone(*node.range, alloc) : nullptr,
        node.closeBracket.deepClone(alloc),
        *deepClone<SequenceExprSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const DelayedSequenceExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DelayedSequenceExprSyntax>(
        node.first ? deepClone(*node.first, alloc) : nullptr,
        *deepClone(node.elements, alloc)
    );
}

SyntaxNode* clone(const DisableConstraintSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DisableConstraintSyntax>(
        node.disable.deepClone(alloc),
        node.soft.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.name, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DisableForkStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DisableForkStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.disable.deepClone(alloc),
        node.fork.deepClone(alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DisableIffSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DisableIffSyntax>(
        node.disable.deepClone(alloc),
        node.iff.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const DisableStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DisableStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.disable.deepClone(alloc),
        *deepClone<NameSyntax>(*node.name, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DistConstraintListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DistConstraintListSyntax>(
        node.dist.deepClone(alloc),
        node.openBrace.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.closeBrace.deepClone(alloc)
    );
}

SyntaxNode* clone(const DistItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DistItemSyntax>(
        *deepClone<ExpressionSyntax>(*node.range, alloc),
        node.weight ? deepClone(*node.weight, alloc) : nullptr
    );
}

SyntaxNode* clone(const DistWeightSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DistWeightSyntax>(
        node.op.deepClone(alloc),
        node.extraOp.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const DividerClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DividerClauseSyntax>(
        node.divide.deepClone(alloc),
        node.value.deepClone(alloc)
    );
}

SyntaxNode* clone(const DoWhileStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DoWhileStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.doKeyword.deepClone(alloc),
        *deepClone<StatementSyntax>(*node.statement, alloc),
        node.whileKeyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const DotMemberClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DotMemberClauseSyntax>(
        node.dot.deepClone(alloc),
        node.member.deepClone(alloc)
    );
}

SyntaxNode* clone(const DriveStrengthSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<DriveStrengthSyntax>(
        node.openParen.deepClone(alloc),
        node.strength0.deepClone(alloc),
        node.comma.deepClone(alloc),
        node.strength1.deepClone(alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const EdgeControlSpecifierSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EdgeControlSpecifierSyntax>(
        node.openBracket.deepClone(alloc),
        *deepClone(node.descriptors, alloc),
        node.closeBracket.deepClone(alloc)
    );
}

SyntaxNode* clone(const EdgeDescriptorSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EdgeDescriptorSyntax>(
        node.t1.deepClone(alloc),
        node.t2.deepClone(alloc)
    );
}

SyntaxNode* clone(const EdgeSensitivePathSuffixSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EdgeSensitivePathSuffixSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone(node.outputs, alloc),
        node.polarityOperator.deepClone(alloc),
        node.colon.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ElabSystemTaskSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ElabSystemTaskSyntax>(
        *deepClone(node.attributes, alloc),
        node.name.deepClone(alloc),
        node.arguments ? deepClone(*node.arguments, alloc) : nullptr,
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ElementSelectExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ElementSelectExpressionSyntax>(
        *deepClone<ExpressionSyntax>(*node.left, alloc),
        *deepClone<ElementSelectSyntax>(*node.select, alloc)
    );
}

SyntaxNode* clone(const ElementSelectSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ElementSelectSyntax>(
        node.openBracket.deepClone(alloc),
        node.selector ? deepClone(*node.selector, alloc) : nullptr,
        node.closeBracket.deepClone(alloc)
    );
}

SyntaxNode* clone(const ElseClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ElseClauseSyntax>(
        node.elseKeyword.deepClone(alloc),
        *deepClone<SyntaxNode>(*node.clause, alloc)
    );
}

SyntaxNode* clone(const ElseConstraintClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ElseConstraintClauseSyntax>(
        node.elseKeyword.deepClone(alloc),
        *deepClone<ConstraintItemSyntax>(*node.constraints, alloc)
    );
}

SyntaxNode* clone(const ElsePropertyClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ElsePropertyClauseSyntax>(
        node.elseKeyword.deepClone(alloc),
        *deepClone<PropertyExprSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const EmptyArgumentSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EmptyArgumentSyntax>(
        node.placeholder.deepClone(alloc)
    );
}

SyntaxNode* clone(const EmptyIdentifierNameSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EmptyIdentifierNameSyntax>(
        node.placeholder.deepClone(alloc)
    );
}

SyntaxNode* clone(const EmptyMemberSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EmptyMemberSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone(node.qualifiers, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const EmptyNonAnsiPortSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EmptyNonAnsiPortSyntax>(
        node.placeholder.deepClone(alloc)
    );
}

SyntaxNode* clone(const EmptyPortConnectionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EmptyPortConnectionSyntax>(
        *deepClone(node.attributes, alloc),
        node.placeholder.deepClone(alloc)
    );
}

SyntaxNode* clone(const EmptyQueueExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EmptyQueueExpressionSyntax>(
        node.openBrace.deepClone(alloc),
        node.closeBrace.deepClone(alloc)
    );
}

SyntaxNode* clone(const EmptyStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EmptyStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.semicolon.deepClone(alloc)
    );
}

SyntaxNode* clone(const EmptyTimingCheckArgSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EmptyTimingCheckArgSyntax>(
        node.placeholder.deepClone(alloc)
    );
}

SyntaxNode* clone(const EnumTypeSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EnumTypeSyntax>(
        node.keyword.deepClone(alloc),
        node.baseType ? deepClone(*node.baseType, alloc) : nullptr,
        node.openBrace.deepClone(alloc),
        *deepClone(node.members, alloc),
        node.closeBrace.deepClone(alloc),
        *deepClone(node.dimensions, alloc)
    );
}

SyntaxNode* clone(const EqualsAssertionArgClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EqualsAssertionArgClauseSyntax>(
        node.equals.deepClone(alloc),
        *deepClone<PropertyExprSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const EqualsTypeClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EqualsTypeClauseSyntax>(
        node.equals.deepClone(alloc),
        *deepClone<DataTypeSyntax>(*node.type, alloc)
    );
}

SyntaxNode* clone(const EqualsValueClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EqualsValueClauseSyntax>(
        node.equals.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const EventControlSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EventControlSyntax>(
        node.at.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.eventName, alloc)
    );
}

SyntaxNode* clone(const EventControlWithExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EventControlWithExpressionSyntax>(
        node.at.deepClone(alloc),
        *deepClone<EventExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const EventTriggerStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<EventTriggerStatementSyntax>(
        node.kind,
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.trigger.deepClone(alloc),
        node.timing ? deepClone(*node.timing, alloc) : nullptr,
        *deepClone<NameSyntax>(*node.name, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ExplicitAnsiPortSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ExplicitAnsiPortSyntax>(
        *deepClone(node.attributes, alloc),
        node.direction.deepClone(alloc),
        node.dot.deepClone(alloc),
        node.name.deepClone(alloc),
        node.openParen.deepClone(alloc),
        node.expr ? deepClone(*node.expr, alloc) : nullptr,
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ExplicitNonAnsiPortSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ExplicitNonAnsiPortSyntax>(
        node.dot.deepClone(alloc),
        node.name.deepClone(alloc),
        node.openParen.deepClone(alloc),
        node.expr ? deepClone(*node.expr, alloc) : nullptr,
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ExpressionConstraintSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ExpressionConstraintSyntax>(
        node.soft.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ExpressionCoverageBinInitializerSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ExpressionCoverageBinInitializerSyntax>(
        *deepClone<ExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const ExpressionOrDistSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ExpressionOrDistSyntax>(
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        *deepClone<DistConstraintListSyntax>(*node.distribution, alloc)
    );
}

SyntaxNode* clone(const ExpressionPatternSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ExpressionPatternSyntax>(
        *deepClone<ExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const ExpressionStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ExpressionStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ExpressionTimingCheckArgSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ExpressionTimingCheckArgSyntax>(
        *deepClone<ExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const ExtendsClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ExtendsClauseSyntax>(
        node.keyword.deepClone(alloc),
        *deepClone<NameSyntax>(*node.baseName, alloc),
        node.arguments ? deepClone(*node.arguments, alloc) : nullptr,
        node.defaultedArg ? deepClone(*node.defaultedArg, alloc) : nullptr
    );
}

SyntaxNode* clone(const ExternInterfaceMethodSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ExternInterfaceMethodSyntax>(
        *deepClone(node.attributes, alloc),
        node.externKeyword.deepClone(alloc),
        node.forkJoin.deepClone(alloc),
        *deepClone<FunctionPrototypeSyntax>(*node.prototype, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ExternModuleDeclSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ExternModuleDeclSyntax>(
        *deepClone(node.attributes, alloc),
        node.externKeyword.deepClone(alloc),
        *deepClone(node.actualAttributes, alloc),
        *deepClone<ModuleHeaderSyntax>(*node.header, alloc)
    );
}

SyntaxNode* clone(const ExternUdpDeclSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ExternUdpDeclSyntax>(
        *deepClone(node.attributes, alloc),
        node.externKeyword.deepClone(alloc),
        *deepClone(node.actualAttributes, alloc),
        node.primitive.deepClone(alloc),
        node.name.deepClone(alloc),
        *deepClone<UdpPortListSyntax>(*node.portList, alloc)
    );
}

SyntaxNode* clone(const FilePathSpecSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<FilePathSpecSyntax>(
        node.path.deepClone(alloc)
    );
}

SyntaxNode* clone(const FirstMatchSequenceExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<FirstMatchSequenceExprSyntax>(
        node.first_match.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<SequenceExprSyntax>(*node.expr, alloc),
        node.matchList ? deepClone(*node.matchList, alloc) : nullptr,
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ForLoopStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ForLoopStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.forKeyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone(node.initializers, alloc),
        node.semi1.deepClone(alloc),
        node.stopExpr ? deepClone(*node.stopExpr, alloc) : nullptr,
        node.semi2.deepClone(alloc),
        *deepClone(node.steps, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone<StatementSyntax>(*node.statement, alloc)
    );
}

SyntaxNode* clone(const ForVariableDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ForVariableDeclarationSyntax>(
        node.varKeyword.deepClone(alloc),
        node.type ? deepClone(*node.type, alloc) : nullptr,
        *deepClone<DeclaratorSyntax>(*node.declarator, alloc)
    );
}

SyntaxNode* clone(const ForeachLoopListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ForeachLoopListSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone<NameSyntax>(*node.arrayName, alloc),
        node.openBracket.deepClone(alloc),
        *deepClone(node.loopVariables, alloc),
        node.closeBracket.deepClone(alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ForeachLoopStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ForeachLoopStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone<ForeachLoopListSyntax>(*node.loopList, alloc),
        *deepClone<StatementSyntax>(*node.statement, alloc)
    );
}

SyntaxNode* clone(const ForeverStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ForeverStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.foreverKeyword.deepClone(alloc),
        *deepClone<StatementSyntax>(*node.statement, alloc)
    );
}

SyntaxNode* clone(const ForwardTypeRestrictionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ForwardTypeRestrictionSyntax>(
        node.keyword1.deepClone(alloc),
        node.keyword2.deepClone(alloc)
    );
}

SyntaxNode* clone(const ForwardTypedefDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ForwardTypedefDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.typedefKeyword.deepClone(alloc),
        node.typeRestriction ? deepClone(*node.typeRestriction, alloc) : nullptr,
        node.name.deepClone(alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const FunctionDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<FunctionDeclarationSyntax>(
        node.kind,
        *deepClone(node.attributes, alloc),
        *deepClone<FunctionPrototypeSyntax>(*node.prototype, alloc),
        node.semi.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.end.deepClone(alloc),
        node.endBlockName ? deepClone(*node.endBlockName, alloc) : nullptr
    );
}

SyntaxNode* clone(const FunctionPortListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<FunctionPortListSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone(node.ports, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const FunctionPortSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<FunctionPortSyntax>(
        *deepClone(node.attributes, alloc),
        node.constKeyword.deepClone(alloc),
        node.direction.deepClone(alloc),
        node.staticKeyword.deepClone(alloc),
        node.varKeyword.deepClone(alloc),
        node.dataType ? deepClone(*node.dataType, alloc) : nullptr,
        *deepClone<DeclaratorSyntax>(*node.declarator, alloc)
    );
}

SyntaxNode* clone(const FunctionPrototypeSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<FunctionPrototypeSyntax>(
        node.keyword.deepClone(alloc),
        *deepClone(node.specifiers, alloc),
        node.lifetime.deepClone(alloc),
        *deepClone<DataTypeSyntax>(*node.returnType, alloc),
        *deepClone<NameSyntax>(*node.name, alloc),
        node.portList ? deepClone(*node.portList, alloc) : nullptr
    );
}

SyntaxNode* clone(const GenerateBlockSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<GenerateBlockSyntax>(
        *deepClone(node.attributes, alloc),
        node.label ? deepClone(*node.label, alloc) : nullptr,
        node.begin.deepClone(alloc),
        node.beginName ? deepClone(*node.beginName, alloc) : nullptr,
        *deepClone(node.members, alloc),
        node.end.deepClone(alloc),
        node.endName ? deepClone(*node.endName, alloc) : nullptr
    );
}

SyntaxNode* clone(const GenerateRegionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<GenerateRegionSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone(node.members, alloc),
        node.endgenerate.deepClone(alloc)
    );
}

SyntaxNode* clone(const GenvarDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<GenvarDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone(node.identifiers, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const HierarchicalInstanceSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<HierarchicalInstanceSyntax>(
        node.decl ? deepClone(*node.decl, alloc) : nullptr,
        node.openParen.deepClone(alloc),
        *deepClone(node.connections, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const HierarchyInstantiationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<HierarchyInstantiationSyntax>(
        *deepClone(node.attributes, alloc),
        node.type.deepClone(alloc),
        node.parameters ? deepClone(*node.parameters, alloc) : nullptr,
        *deepClone(node.instances, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const IdWithExprCoverageBinInitializerSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<IdWithExprCoverageBinInitializerSyntax>(
        node.id.deepClone(alloc),
        *deepClone<WithClauseSyntax>(*node.withClause, alloc)
    );
}

SyntaxNode* clone(const IdentifierNameSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<IdentifierNameSyntax>(
        node.identifier.deepClone(alloc)
    );
}

SyntaxNode* clone(const IdentifierSelectNameSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<IdentifierSelectNameSyntax>(
        node.identifier.deepClone(alloc),
        *deepClone(node.selectors, alloc)
    );
}

SyntaxNode* clone(const IfGenerateSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<IfGenerateSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.condition, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone<MemberSyntax>(*node.block, alloc),
        node.elseClause ? deepClone(*node.elseClause, alloc) : nullptr
    );
}

SyntaxNode* clone(const IfNonePathDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<IfNonePathDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone<PathDeclarationSyntax>(*node.path, alloc)
    );
}

SyntaxNode* clone(const IffEventClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<IffEventClauseSyntax>(
        node.iff.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const ImmediateAssertionMemberSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ImmediateAssertionMemberSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone<ImmediateAssertionStatementSyntax>(*node.statement, alloc)
    );
}

SyntaxNode* clone(const ImmediateAssertionStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ImmediateAssertionStatementSyntax>(
        node.kind,
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.delay ? deepClone(*node.delay, alloc) : nullptr,
        *deepClone<ParenthesizedExpressionSyntax>(*node.expr, alloc),
        *deepClone<ActionBlockSyntax>(*node.action, alloc)
    );
}

SyntaxNode* clone(const ImplementsClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ImplementsClauseSyntax>(
        node.keyword.deepClone(alloc),
        *deepClone(node.interfaces, alloc)
    );
}

SyntaxNode* clone(const ImplicationConstraintSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ImplicationConstraintSyntax>(
        *deepClone<ExpressionSyntax>(*node.left, alloc),
        node.arrow.deepClone(alloc),
        *deepClone<ConstraintItemSyntax>(*node.constraints, alloc)
    );
}

SyntaxNode* clone(const ImplicitAnsiPortSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ImplicitAnsiPortSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone<PortHeaderSyntax>(*node.header, alloc),
        *deepClone<DeclaratorSyntax>(*node.declarator, alloc)
    );
}

SyntaxNode* clone(const ImplicitEventControlSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ImplicitEventControlSyntax>(
        node.at.deepClone(alloc),
        node.openParen.deepClone(alloc),
        node.star.deepClone(alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ImplicitNonAnsiPortSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ImplicitNonAnsiPortSyntax>(
        *deepClone<PortExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const ImplicitTypeSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ImplicitTypeSyntax>(
        node.signing.deepClone(alloc),
        *deepClone(node.dimensions, alloc),
        node.placeholder.deepClone(alloc)
    );
}

SyntaxNode* clone(const IncludeDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<IncludeDirectiveSyntax>(
        node.directive.deepClone(alloc),
        node.fileName.deepClone(alloc)
    );
}

SyntaxNode* clone(const InsideExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<InsideExpressionSyntax>(
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.inside.deepClone(alloc),
        *deepClone<RangeListSyntax>(*node.ranges, alloc)
    );
}

SyntaxNode* clone(const InstanceConfigRuleSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<InstanceConfigRuleSyntax>(
        node.instance.deepClone(alloc),
        node.topModule.deepClone(alloc),
        *deepClone(node.instanceNames, alloc),
        *deepClone<ConfigRuleClauseSyntax>(*node.ruleClause, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const InstanceNameSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<InstanceNameSyntax>(
        node.name.deepClone(alloc),
        *deepClone(node.dimensions, alloc)
    );
}

SyntaxNode* clone(const IntegerTypeSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<IntegerTypeSyntax>(
        node.kind,
        node.keyword.deepClone(alloc),
        node.signing.deepClone(alloc),
        *deepClone(node.dimensions, alloc)
    );
}

SyntaxNode* clone(const IntegerVectorExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<IntegerVectorExpressionSyntax>(
        node.size.deepClone(alloc),
        node.base.deepClone(alloc),
        node.value.deepClone(alloc)
    );
}

SyntaxNode* clone(const InterfacePortHeaderSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<InterfacePortHeaderSyntax>(
        node.nameOrKeyword.deepClone(alloc),
        node.modport ? deepClone(*node.modport, alloc) : nullptr
    );
}

SyntaxNode* clone(const IntersectClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<IntersectClauseSyntax>(
        node.intersect.deepClone(alloc),
        *deepClone<RangeListSyntax>(*node.ranges, alloc)
    );
}

SyntaxNode* clone(const InvocationExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<InvocationExpressionSyntax>(
        *deepClone<ExpressionSyntax>(*node.left, alloc),
        *deepClone(node.attributes, alloc),
        node.arguments ? deepClone(*node.arguments, alloc) : nullptr
    );
}

SyntaxNode* clone(const JumpStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<JumpStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.breakOrContinue.deepClone(alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const KeywordNameSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<KeywordNameSyntax>(
        node.kind,
        node.keyword.deepClone(alloc)
    );
}

SyntaxNode* clone(const KeywordTypeSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<KeywordTypeSyntax>(
        node.kind,
        node.keyword.deepClone(alloc)
    );
}

SyntaxNode* clone(const LetDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<LetDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.let.deepClone(alloc),
        node.identifier.deepClone(alloc),
        node.portList ? deepClone(*node.portList, alloc) : nullptr,
        node.equals.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const LibraryDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<LibraryDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.library.deepClone(alloc),
        node.name.deepClone(alloc),
        *deepClone(node.filePaths, alloc),
        node.incDirClause ? deepClone(*node.incDirClause, alloc) : nullptr,
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const LibraryIncDirClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<LibraryIncDirClauseSyntax>(
        node.minus.deepClone(alloc),
        node.incdir.deepClone(alloc),
        *deepClone(node.filePaths, alloc)
    );
}

SyntaxNode* clone(const LibraryIncludeStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<LibraryIncludeStatementSyntax>(
        *deepClone(node.attributes, alloc),
        node.include.deepClone(alloc),
        *deepClone<FilePathSpecSyntax>(*node.filePath, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const LibraryMapSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<LibraryMapSyntax>(
        *deepClone(node.members, alloc),
        node.endOfFile.deepClone(alloc)
    );
}

SyntaxNode* clone(const LineDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<LineDirectiveSyntax>(
        node.directive.deepClone(alloc),
        node.lineNumber.deepClone(alloc),
        node.fileName.deepClone(alloc),
        node.level.deepClone(alloc)
    );
}

SyntaxNode* clone(const LiteralExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<LiteralExpressionSyntax>(
        node.kind,
        node.literal.deepClone(alloc)
    );
}

SyntaxNode* clone(const LocalVariableDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<LocalVariableDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.var.deepClone(alloc),
        *deepClone<DataTypeSyntax>(*node.type, alloc),
        *deepClone(node.declarators, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const LoopConstraintSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<LoopConstraintSyntax>(
        node.foreachKeyword.deepClone(alloc),
        *deepClone<ForeachLoopListSyntax>(*node.loopList, alloc),
        *deepClone<ConstraintItemSyntax>(*node.constraints, alloc)
    );
}

SyntaxNode* clone(const LoopGenerateSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<LoopGenerateSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        node.genvar.deepClone(alloc),
        node.identifier.deepClone(alloc),
        node.equals.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.initialExpr, alloc),
        node.semi1.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.stopExpr, alloc),
        node.semi2.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.iterationExpr, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone<MemberSyntax>(*node.block, alloc)
    );
}

SyntaxNode* clone(const LoopStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<LoopStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.repeatOrWhile.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone<StatementSyntax>(*node.statement, alloc)
    );
}

SyntaxNode* clone(const MacroActualArgumentListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<MacroActualArgumentListSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone(node.args, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const MacroActualArgumentSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<MacroActualArgumentSyntax>(
        *deepClone(node.tokens, alloc)
    );
}

SyntaxNode* clone(const MacroArgumentDefaultSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<MacroArgumentDefaultSyntax>(
        node.equals.deepClone(alloc),
        *deepClone(node.tokens, alloc)
    );
}

SyntaxNode* clone(const MacroFormalArgumentListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<MacroFormalArgumentListSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone(node.args, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const MacroFormalArgumentSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<MacroFormalArgumentSyntax>(
        node.name.deepClone(alloc),
        node.defaultValue ? deepClone(*node.defaultValue, alloc) : nullptr
    );
}

SyntaxNode* clone(const MacroUsageSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<MacroUsageSyntax>(
        node.directive.deepClone(alloc),
        node.args ? deepClone(*node.args, alloc) : nullptr
    );
}

SyntaxNode* clone(const MatchesClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<MatchesClauseSyntax>(
        node.matchesKeyword.deepClone(alloc),
        *deepClone<PatternSyntax>(*node.pattern, alloc)
    );
}

SyntaxNode* clone(const MemberAccessExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<MemberAccessExpressionSyntax>(
        *deepClone<ExpressionSyntax>(*node.left, alloc),
        node.dot.deepClone(alloc),
        node.name.deepClone(alloc)
    );
}

SyntaxNode* clone(const MinTypMaxExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<MinTypMaxExpressionSyntax>(
        *deepClone<ExpressionSyntax>(*node.min, alloc),
        node.colon1.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.typ, alloc),
        node.colon2.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.max, alloc)
    );
}

SyntaxNode* clone(const ModportClockingPortSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ModportClockingPortSyntax>(
        *deepClone(node.attributes, alloc),
        node.clocking.deepClone(alloc),
        node.name.deepClone(alloc)
    );
}

SyntaxNode* clone(const ModportDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ModportDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ModportExplicitPortSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ModportExplicitPortSyntax>(
        node.dot.deepClone(alloc),
        node.name.deepClone(alloc),
        node.openParen.deepClone(alloc),
        node.expr ? deepClone(*node.expr, alloc) : nullptr,
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ModportItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ModportItemSyntax>(
        node.name.deepClone(alloc),
        *deepClone<AnsiPortListSyntax>(*node.ports, alloc)
    );
}

SyntaxNode* clone(const ModportNamedPortSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ModportNamedPortSyntax>(
        node.name.deepClone(alloc)
    );
}

SyntaxNode* clone(const ModportSimplePortListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ModportSimplePortListSyntax>(
        *deepClone(node.attributes, alloc),
        node.direction.deepClone(alloc),
        *deepClone(node.ports, alloc)
    );
}

SyntaxNode* clone(const ModportSubroutinePortListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ModportSubroutinePortListSyntax>(
        *deepClone(node.attributes, alloc),
        node.importExport.deepClone(alloc),
        *deepClone(node.ports, alloc)
    );
}

SyntaxNode* clone(const ModportSubroutinePortSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ModportSubroutinePortSyntax>(
        *deepClone<FunctionPrototypeSyntax>(*node.prototype, alloc)
    );
}

SyntaxNode* clone(const ModuleDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ModuleDeclarationSyntax>(
        node.kind,
        *deepClone(node.attributes, alloc),
        *deepClone<ModuleHeaderSyntax>(*node.header, alloc),
        *deepClone(node.members, alloc),
        node.endmodule.deepClone(alloc),
        node.blockName ? deepClone(*node.blockName, alloc) : nullptr
    );
}

SyntaxNode* clone(const ModuleHeaderSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ModuleHeaderSyntax>(
        node.kind,
        node.moduleKeyword.deepClone(alloc),
        node.lifetime.deepClone(alloc),
        node.name.deepClone(alloc),
        *deepClone(node.imports, alloc),
        node.parameters ? deepClone(*node.parameters, alloc) : nullptr,
        node.ports ? deepClone(*node.ports, alloc) : nullptr,
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const MultipleConcatenationExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<MultipleConcatenationExpressionSyntax>(
        node.openBrace.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expression, alloc),
        *deepClone<ConcatenationExpressionSyntax>(*node.concatenation, alloc),
        node.closeBrace.deepClone(alloc)
    );
}

SyntaxNode* clone(const NameValuePragmaExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NameValuePragmaExpressionSyntax>(
        node.name.deepClone(alloc),
        node.equals.deepClone(alloc),
        *deepClone<PragmaExpressionSyntax>(*node.value, alloc)
    );
}

SyntaxNode* clone(const NamedArgumentSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NamedArgumentSyntax>(
        node.dot.deepClone(alloc),
        node.name.deepClone(alloc),
        node.openParen.deepClone(alloc),
        node.expr ? deepClone(*node.expr, alloc) : nullptr,
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const NamedBlockClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NamedBlockClauseSyntax>(
        node.colon.deepClone(alloc),
        node.name.deepClone(alloc)
    );
}

SyntaxNode* clone(const NamedConditionalDirectiveExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NamedConditionalDirectiveExpressionSyntax>(
        node.name.deepClone(alloc)
    );
}

SyntaxNode* clone(const NamedLabelSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NamedLabelSyntax>(
        node.name.deepClone(alloc),
        node.colon.deepClone(alloc)
    );
}

SyntaxNode* clone(const NamedParamAssignmentSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NamedParamAssignmentSyntax>(
        node.dot.deepClone(alloc),
        node.name.deepClone(alloc),
        node.openParen.deepClone(alloc),
        node.expr ? deepClone(*node.expr, alloc) : nullptr,
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const NamedPortConnectionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NamedPortConnectionSyntax>(
        *deepClone(node.attributes, alloc),
        node.dot.deepClone(alloc),
        node.name.deepClone(alloc),
        node.openParen.deepClone(alloc),
        node.expr ? deepClone(*node.expr, alloc) : nullptr,
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const NamedStructurePatternMemberSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NamedStructurePatternMemberSyntax>(
        node.name.deepClone(alloc),
        node.colon.deepClone(alloc),
        *deepClone<PatternSyntax>(*node.pattern, alloc)
    );
}

SyntaxNode* clone(const NamedTypeSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NamedTypeSyntax>(
        *deepClone<NameSyntax>(*node.name, alloc)
    );
}

SyntaxNode* clone(const NetAliasSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NetAliasSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone(node.nets, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const NetDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NetDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.netType.deepClone(alloc),
        node.strength ? deepClone(*node.strength, alloc) : nullptr,
        node.expansionHint.deepClone(alloc),
        *deepClone<DataTypeSyntax>(*node.type, alloc),
        node.delay ? deepClone(*node.delay, alloc) : nullptr,
        *deepClone(node.declarators, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const NetPortHeaderSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NetPortHeaderSyntax>(
        node.direction.deepClone(alloc),
        node.netType.deepClone(alloc),
        *deepClone<DataTypeSyntax>(*node.dataType, alloc)
    );
}

SyntaxNode* clone(const NetTypeDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NetTypeDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone<DataTypeSyntax>(*node.type, alloc),
        node.name.deepClone(alloc),
        node.withFunction ? deepClone(*node.withFunction, alloc) : nullptr,
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const NewArrayExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NewArrayExpressionSyntax>(
        *deepClone<NameSyntax>(*node.newKeyword, alloc),
        node.openBracket.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.sizeExpr, alloc),
        node.closeBracket.deepClone(alloc),
        node.initializer ? deepClone(*node.initializer, alloc) : nullptr
    );
}

SyntaxNode* clone(const NewClassExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NewClassExpressionSyntax>(
        *deepClone<NameSyntax>(*node.scopedNew, alloc),
        node.argList ? deepClone(*node.argList, alloc) : nullptr
    );
}

SyntaxNode* clone(const NonAnsiPortListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NonAnsiPortListSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone(node.ports, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const NonAnsiUdpPortListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NonAnsiUdpPortListSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone(node.ports, alloc),
        node.closeParen.deepClone(alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const NumberPragmaExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<NumberPragmaExpressionSyntax>(
        node.size.deepClone(alloc),
        node.base.deepClone(alloc),
        node.value.deepClone(alloc)
    );
}

SyntaxNode* clone(const OneStepDelaySyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<OneStepDelaySyntax>(
        node.hash.deepClone(alloc),
        node.oneStep.deepClone(alloc)
    );
}

SyntaxNode* clone(const OrderedArgumentSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<OrderedArgumentSyntax>(
        *deepClone<PropertyExprSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const OrderedParamAssignmentSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<OrderedParamAssignmentSyntax>(
        *deepClone<ExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const OrderedPortConnectionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<OrderedPortConnectionSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone<PropertyExprSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const OrderedStructurePatternMemberSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<OrderedStructurePatternMemberSyntax>(
        *deepClone<PatternSyntax>(*node.pattern, alloc)
    );
}

SyntaxNode* clone(const PackageExportAllDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PackageExportAllDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.star1.deepClone(alloc),
        node.doubleColon.deepClone(alloc),
        node.star2.deepClone(alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const PackageExportDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PackageExportDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const PackageImportDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PackageImportDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const PackageImportItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PackageImportItemSyntax>(
        node.package.deepClone(alloc),
        node.doubleColon.deepClone(alloc),
        node.item.deepClone(alloc)
    );
}

SyntaxNode* clone(const ParameterDeclarationStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ParameterDeclarationStatementSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone<ParameterDeclarationBaseSyntax>(*node.parameter, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ParameterDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ParameterDeclarationSyntax>(
        node.keyword.deepClone(alloc),
        *deepClone<DataTypeSyntax>(*node.type, alloc),
        *deepClone(node.declarators, alloc)
    );
}

SyntaxNode* clone(const ParameterPortListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ParameterPortListSyntax>(
        node.hash.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone(node.declarations, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ParameterValueAssignmentSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ParameterValueAssignmentSyntax>(
        node.hash.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone(node.parameters, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ParenExpressionListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ParenExpressionListSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone(node.expressions, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ParenPragmaExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ParenPragmaExpressionSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone(node.values, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ParenthesizedBinsSelectExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ParenthesizedBinsSelectExprSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone<BinsSelectExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ParenthesizedConditionalDirectiveExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ParenthesizedConditionalDirectiveExpressionSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone<ConditionalDirectiveExpressionSyntax>(*node.operand, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ParenthesizedEventExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ParenthesizedEventExpressionSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone<EventExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ParenthesizedExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ParenthesizedExpressionSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expression, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ParenthesizedPatternSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ParenthesizedPatternSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone<PatternSyntax>(*node.pattern, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ParenthesizedPropertyExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ParenthesizedPropertyExprSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone<PropertyExprSyntax>(*node.expr, alloc),
        node.matchList ? deepClone(*node.matchList, alloc) : nullptr,
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const ParenthesizedSequenceExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ParenthesizedSequenceExprSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone<SequenceExprSyntax>(*node.expr, alloc),
        node.matchList ? deepClone(*node.matchList, alloc) : nullptr,
        node.closeParen.deepClone(alloc),
        node.repetition ? deepClone(*node.repetition, alloc) : nullptr
    );
}

SyntaxNode* clone(const PathDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PathDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone<PathDescriptionSyntax>(*node.desc, alloc),
        node.equals.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone(node.delays, alloc),
        node.closeParen.deepClone(alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const PathDescriptionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PathDescriptionSyntax>(
        node.openParen.deepClone(alloc),
        node.edgeIdentifier.deepClone(alloc),
        *deepClone(node.inputs, alloc),
        node.polarityOperator.deepClone(alloc),
        node.pathOperator.deepClone(alloc),
        *deepClone<PathSuffixSyntax>(*node.suffix, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const PatternCaseItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PatternCaseItemSyntax>(
        *deepClone<PatternSyntax>(*node.pattern, alloc),
        node.tripleAnd.deepClone(alloc),
        node.expr ? deepClone(*node.expr, alloc) : nullptr,
        node.colon.deepClone(alloc),
        *deepClone<StatementSyntax>(*node.statement, alloc)
    );
}

SyntaxNode* clone(const PortConcatenationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PortConcatenationSyntax>(
        node.openBrace.deepClone(alloc),
        *deepClone(node.references, alloc),
        node.closeBrace.deepClone(alloc)
    );
}

SyntaxNode* clone(const PortDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PortDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        *deepClone<PortHeaderSyntax>(*node.header, alloc),
        *deepClone(node.declarators, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const PortReferenceSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PortReferenceSyntax>(
        node.name.deepClone(alloc),
        node.select ? deepClone(*node.select, alloc) : nullptr
    );
}

SyntaxNode* clone(const PostfixUnaryExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PostfixUnaryExpressionSyntax>(
        node.kind,
        *deepClone<ExpressionSyntax>(*node.operand, alloc),
        *deepClone(node.attributes, alloc),
        node.operatorToken.deepClone(alloc)
    );
}

SyntaxNode* clone(const PragmaDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PragmaDirectiveSyntax>(
        node.directive.deepClone(alloc),
        node.name.deepClone(alloc),
        *deepClone(node.args, alloc)
    );
}

SyntaxNode* clone(const PrefixUnaryExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PrefixUnaryExpressionSyntax>(
        node.kind,
        node.operatorToken.deepClone(alloc),
        *deepClone(node.attributes, alloc),
        *deepClone<ExpressionSyntax>(*node.operand, alloc)
    );
}

SyntaxNode* clone(const PrimaryBlockEventExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PrimaryBlockEventExpressionSyntax>(
        node.keyword.deepClone(alloc),
        *deepClone<NameSyntax>(*node.name, alloc)
    );
}

SyntaxNode* clone(const PrimitiveInstantiationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PrimitiveInstantiationSyntax>(
        *deepClone(node.attributes, alloc),
        node.type.deepClone(alloc),
        node.strength ? deepClone(*node.strength, alloc) : nullptr,
        node.delay ? deepClone(*node.delay, alloc) : nullptr,
        *deepClone(node.instances, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ProceduralAssignStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ProceduralAssignStatementSyntax>(
        node.kind,
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ProceduralBlockSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ProceduralBlockSyntax>(
        node.kind,
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone<StatementSyntax>(*node.statement, alloc)
    );
}

SyntaxNode* clone(const ProceduralDeassignStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ProceduralDeassignStatementSyntax>(
        node.kind,
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.variable, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ProductionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ProductionSyntax>(
        node.dataType ? deepClone(*node.dataType, alloc) : nullptr,
        node.name.deepClone(alloc),
        node.portList ? deepClone(*node.portList, alloc) : nullptr,
        node.colon.deepClone(alloc),
        *deepClone(node.rules, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const PropertyDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PropertyDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.name.deepClone(alloc),
        node.portList ? deepClone(*node.portList, alloc) : nullptr,
        node.semi.deepClone(alloc),
        *deepClone(node.variables, alloc),
        *deepClone<PropertySpecSyntax>(*node.propertySpec, alloc),
        node.optionalSemi.deepClone(alloc),
        node.end.deepClone(alloc),
        node.endBlockName ? deepClone(*node.endBlockName, alloc) : nullptr
    );
}

SyntaxNode* clone(const PropertySpecSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PropertySpecSyntax>(
        node.clocking ? deepClone(*node.clocking, alloc) : nullptr,
        node.disable ? deepClone(*node.disable, alloc) : nullptr,
        *deepClone<PropertyExprSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const PullStrengthSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PullStrengthSyntax>(
        node.openParen.deepClone(alloc),
        node.strength.deepClone(alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const PulseStyleDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<PulseStyleDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone(node.inputs, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const QueueDimensionSpecifierSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<QueueDimensionSpecifierSyntax>(
        node.dollar.deepClone(alloc),
        node.maxSizeClause ? deepClone(*node.maxSizeClause, alloc) : nullptr
    );
}

SyntaxNode* clone(const RandCaseItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RandCaseItemSyntax>(
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.colon.deepClone(alloc),
        *deepClone<StatementSyntax>(*node.statement, alloc)
    );
}

SyntaxNode* clone(const RandCaseStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RandCaseStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.randCase.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.endCase.deepClone(alloc)
    );
}

SyntaxNode* clone(const RandJoinClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RandJoinClauseSyntax>(
        node.rand.deepClone(alloc),
        node.join.deepClone(alloc),
        node.expr ? deepClone(*node.expr, alloc) : nullptr
    );
}

SyntaxNode* clone(const RandSequenceStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RandSequenceStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.randsequence.deepClone(alloc),
        node.openParen.deepClone(alloc),
        node.firstProduction.deepClone(alloc),
        node.closeParen.deepClone(alloc),
        *deepClone(node.productions, alloc),
        node.endsequence.deepClone(alloc)
    );
}

SyntaxNode* clone(const RangeCoverageBinInitializerSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RangeCoverageBinInitializerSyntax>(
        *deepClone<RangeListSyntax>(*node.ranges, alloc),
        node.withClause ? deepClone(*node.withClause, alloc) : nullptr
    );
}

SyntaxNode* clone(const RangeDimensionSpecifierSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RangeDimensionSpecifierSyntax>(
        *deepClone<SelectorSyntax>(*node.selector, alloc)
    );
}

SyntaxNode* clone(const RangeListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RangeListSyntax>(
        node.openBrace.deepClone(alloc),
        *deepClone(node.valueRanges, alloc),
        node.closeBrace.deepClone(alloc)
    );
}

SyntaxNode* clone(const RangeSelectSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RangeSelectSyntax>(
        node.kind,
        *deepClone<ExpressionSyntax>(*node.left, alloc),
        node.range.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.right, alloc)
    );
}

SyntaxNode* clone(const RepeatedEventControlSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RepeatedEventControlSyntax>(
        node.repeat.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc),
        node.eventControl ? deepClone(*node.eventControl, alloc) : nullptr
    );
}

SyntaxNode* clone(const ReplicatedAssignmentPatternSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ReplicatedAssignmentPatternSyntax>(
        node.openBrace.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.countExpr, alloc),
        node.innerOpenBrace.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.innerCloseBrace.deepClone(alloc),
        node.closeBrace.deepClone(alloc)
    );
}

SyntaxNode* clone(const ReturnStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ReturnStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.returnKeyword.deepClone(alloc),
        node.returnValue ? deepClone(*node.returnValue, alloc) : nullptr,
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const RsCaseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RsCaseSyntax>(
        node.keyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.endcase.deepClone(alloc)
    );
}

SyntaxNode* clone(const RsCodeBlockSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RsCodeBlockSyntax>(
        node.openBrace.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.closeBrace.deepClone(alloc)
    );
}

SyntaxNode* clone(const RsElseClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RsElseClauseSyntax>(
        node.keyword.deepClone(alloc),
        *deepClone<RsProdItemSyntax>(*node.item, alloc)
    );
}

SyntaxNode* clone(const RsIfElseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RsIfElseSyntax>(
        node.keyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.condition, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone<RsProdItemSyntax>(*node.ifItem, alloc),
        node.elseClause ? deepClone(*node.elseClause, alloc) : nullptr
    );
}

SyntaxNode* clone(const RsProdItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RsProdItemSyntax>(
        node.name.deepClone(alloc),
        node.argList ? deepClone(*node.argList, alloc) : nullptr
    );
}

SyntaxNode* clone(const RsRepeatSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RsRepeatSyntax>(
        node.keyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone<RsProdItemSyntax>(*node.item, alloc)
    );
}

SyntaxNode* clone(const RsRuleSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RsRuleSyntax>(
        node.randJoin ? deepClone(*node.randJoin, alloc) : nullptr,
        *deepClone(node.prods, alloc),
        node.weightClause ? deepClone(*node.weightClause, alloc) : nullptr
    );
}

SyntaxNode* clone(const RsWeightClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<RsWeightClauseSyntax>(
        node.colonEqual.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.weight, alloc),
        node.codeBlock ? deepClone(*node.codeBlock, alloc) : nullptr
    );
}

SyntaxNode* clone(const ScopedNameSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ScopedNameSyntax>(
        *deepClone<NameSyntax>(*node.left, alloc),
        node.separator.deepClone(alloc),
        *deepClone<NameSyntax>(*node.right, alloc)
    );
}

SyntaxNode* clone(const SequenceDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SequenceDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.name.deepClone(alloc),
        node.portList ? deepClone(*node.portList, alloc) : nullptr,
        node.semi.deepClone(alloc),
        *deepClone(node.variables, alloc),
        *deepClone<SequenceExprSyntax>(*node.seqExpr, alloc),
        node.optionalSemi.deepClone(alloc),
        node.end.deepClone(alloc),
        node.endBlockName ? deepClone(*node.endBlockName, alloc) : nullptr
    );
}

SyntaxNode* clone(const SequenceMatchListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SequenceMatchListSyntax>(
        node.comma.deepClone(alloc),
        *deepClone(node.items, alloc)
    );
}

SyntaxNode* clone(const SequenceRepetitionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SequenceRepetitionSyntax>(
        node.openBracket.deepClone(alloc),
        node.op.deepClone(alloc),
        node.selector ? deepClone(*node.selector, alloc) : nullptr,
        node.closeBracket.deepClone(alloc)
    );
}

SyntaxNode* clone(const SignalEventExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SignalEventExpressionSyntax>(
        node.edge.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.iffClause ? deepClone(*node.iffClause, alloc) : nullptr
    );
}

SyntaxNode* clone(const SignedCastExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SignedCastExpressionSyntax>(
        node.signing.deepClone(alloc),
        node.apostrophe.deepClone(alloc),
        *deepClone<ParenthesizedExpressionSyntax>(*node.inner, alloc)
    );
}

SyntaxNode* clone(const SimpleAssignmentPatternSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SimpleAssignmentPatternSyntax>(
        node.openBrace.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.closeBrace.deepClone(alloc)
    );
}

SyntaxNode* clone(const SimpleBinsSelectExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SimpleBinsSelectExprSyntax>(
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.matchesClause ? deepClone(*node.matchesClause, alloc) : nullptr
    );
}

SyntaxNode* clone(const SimpleDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SimpleDirectiveSyntax>(
        node.kind,
        node.directive.deepClone(alloc)
    );
}

SyntaxNode* clone(const SimplePathSuffixSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SimplePathSuffixSyntax>(
        *deepClone(node.outputs, alloc)
    );
}

SyntaxNode* clone(const SimplePragmaExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SimplePragmaExpressionSyntax>(
        node.value.deepClone(alloc)
    );
}

SyntaxNode* clone(const SimplePropertyExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SimplePropertyExprSyntax>(
        *deepClone<SequenceExprSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const SimpleSequenceExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SimpleSequenceExprSyntax>(
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.repetition ? deepClone(*node.repetition, alloc) : nullptr
    );
}

SyntaxNode* clone(const SolveBeforeConstraintSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SolveBeforeConstraintSyntax>(
        node.solve.deepClone(alloc),
        *deepClone(node.beforeExpr, alloc),
        node.before.deepClone(alloc),
        *deepClone(node.afterExpr, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const SpecifyBlockSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SpecifyBlockSyntax>(
        *deepClone(node.attributes, alloc),
        node.specify.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.endspecify.deepClone(alloc)
    );
}

SyntaxNode* clone(const SpecparamDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SpecparamDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone<ImplicitTypeSyntax>(*node.type, alloc),
        *deepClone(node.declarators, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const SpecparamDeclaratorSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SpecparamDeclaratorSyntax>(
        node.name.deepClone(alloc),
        node.equals.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.value1, alloc),
        node.comma.deepClone(alloc),
        node.value2 ? deepClone(*node.value2, alloc) : nullptr,
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const StandardCaseItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<StandardCaseItemSyntax>(
        *deepClone(node.expressions, alloc),
        node.colon.deepClone(alloc),
        *deepClone<SyntaxNode>(*node.clause, alloc)
    );
}

SyntaxNode* clone(const StandardPropertyCaseItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<StandardPropertyCaseItemSyntax>(
        *deepClone(node.expressions, alloc),
        node.colon.deepClone(alloc),
        *deepClone<PropertyExprSyntax>(*node.expr, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const StandardRsCaseItemSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<StandardRsCaseItemSyntax>(
        *deepClone(node.expressions, alloc),
        node.colon.deepClone(alloc),
        *deepClone<RsProdItemSyntax>(*node.item, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const StreamExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<StreamExpressionSyntax>(
        *deepClone<ExpressionSyntax>(*node.expression, alloc),
        node.withRange ? deepClone(*node.withRange, alloc) : nullptr
    );
}

SyntaxNode* clone(const StreamExpressionWithRangeSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<StreamExpressionWithRangeSyntax>(
        node.withKeyword.deepClone(alloc),
        *deepClone<ElementSelectSyntax>(*node.range, alloc)
    );
}

SyntaxNode* clone(const StreamingConcatenationExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<StreamingConcatenationExpressionSyntax>(
        node.openBrace.deepClone(alloc),
        node.operatorToken.deepClone(alloc),
        node.sliceSize ? deepClone(*node.sliceSize, alloc) : nullptr,
        node.innerOpenBrace.deepClone(alloc),
        *deepClone(node.expressions, alloc),
        node.innerCloseBrace.deepClone(alloc),
        node.closeBrace.deepClone(alloc)
    );
}

SyntaxNode* clone(const StrongWeakPropertyExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<StrongWeakPropertyExprSyntax>(
        node.keyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<SequenceExprSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const StructUnionMemberSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<StructUnionMemberSyntax>(
        *deepClone(node.attributes, alloc),
        node.randomQualifier.deepClone(alloc),
        *deepClone<DataTypeSyntax>(*node.type, alloc),
        *deepClone(node.declarators, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const StructUnionTypeSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<StructUnionTypeSyntax>(
        node.kind,
        node.keyword.deepClone(alloc),
        node.taggedOrSoft.deepClone(alloc),
        node.packed.deepClone(alloc),
        node.signing.deepClone(alloc),
        node.openBrace.deepClone(alloc),
        *deepClone(node.members, alloc),
        node.closeBrace.deepClone(alloc),
        *deepClone(node.dimensions, alloc)
    );
}

SyntaxNode* clone(const StructurePatternSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<StructurePatternSyntax>(
        node.openBrace.deepClone(alloc),
        *deepClone(node.members, alloc),
        node.closeBrace.deepClone(alloc)
    );
}

SyntaxNode* clone(const StructuredAssignmentPatternSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<StructuredAssignmentPatternSyntax>(
        node.openBrace.deepClone(alloc),
        *deepClone(node.items, alloc),
        node.closeBrace.deepClone(alloc)
    );
}

SyntaxNode* clone(const SuperNewDefaultedArgsExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SuperNewDefaultedArgsExpressionSyntax>(
        *deepClone<NameSyntax>(*node.scopedNew, alloc),
        node.openParen.deepClone(alloc),
        node.defaultKeyword.deepClone(alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const SystemNameSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SystemNameSyntax>(
        node.systemIdentifier.deepClone(alloc)
    );
}

SyntaxNode* clone(const SystemTimingCheckSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<SystemTimingCheckSyntax>(
        *deepClone(node.attributes, alloc),
        node.name.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone(node.args, alloc),
        node.closeParen.deepClone(alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const TaggedPatternSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TaggedPatternSyntax>(
        node.tagged.deepClone(alloc),
        node.memberName.deepClone(alloc),
        node.pattern ? deepClone(*node.pattern, alloc) : nullptr
    );
}

SyntaxNode* clone(const TaggedUnionExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TaggedUnionExpressionSyntax>(
        node.tagged.deepClone(alloc),
        node.member.deepClone(alloc),
        node.expr ? deepClone(*node.expr, alloc) : nullptr
    );
}

SyntaxNode* clone(const TimeScaleDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TimeScaleDirectiveSyntax>(
        node.directive.deepClone(alloc),
        node.timeUnit.deepClone(alloc),
        node.slash.deepClone(alloc),
        node.timePrecision.deepClone(alloc)
    );
}

SyntaxNode* clone(const TimeUnitsDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TimeUnitsDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.time.deepClone(alloc),
        node.divider ? deepClone(*node.divider, alloc) : nullptr,
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const TimingCheckEventArgSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TimingCheckEventArgSyntax>(
        node.edge.deepClone(alloc),
        node.controlSpecifier ? deepClone(*node.controlSpecifier, alloc) : nullptr,
        *deepClone<ExpressionSyntax>(*node.terminal, alloc),
        node.condition ? deepClone(*node.condition, alloc) : nullptr
    );
}

SyntaxNode* clone(const TimingCheckEventConditionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TimingCheckEventConditionSyntax>(
        node.tripleAnd.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const TimingControlExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TimingControlExpressionSyntax>(
        *deepClone<TimingControlSyntax>(*node.timing, alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const TimingControlStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TimingControlStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        *deepClone<TimingControlSyntax>(*node.timingControl, alloc),
        *deepClone<StatementSyntax>(*node.statement, alloc)
    );
}

SyntaxNode* clone(const TransListCoverageBinInitializerSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TransListCoverageBinInitializerSyntax>(
        *deepClone(node.sets, alloc)
    );
}

SyntaxNode* clone(const TransRangeSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TransRangeSyntax>(
        *deepClone(node.items, alloc),
        node.repeat ? deepClone(*node.repeat, alloc) : nullptr
    );
}

SyntaxNode* clone(const TransRepeatRangeSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TransRepeatRangeSyntax>(
        node.openBracket.deepClone(alloc),
        node.specifier.deepClone(alloc),
        node.selector ? deepClone(*node.selector, alloc) : nullptr,
        node.closeBracket.deepClone(alloc)
    );
}

SyntaxNode* clone(const TransSetSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TransSetSyntax>(
        node.openParen.deepClone(alloc),
        *deepClone(node.ranges, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const TypeAssignmentSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TypeAssignmentSyntax>(
        node.name.deepClone(alloc),
        node.assignment ? deepClone(*node.assignment, alloc) : nullptr
    );
}

SyntaxNode* clone(const TypeParameterDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TypeParameterDeclarationSyntax>(
        node.keyword.deepClone(alloc),
        node.typeKeyword.deepClone(alloc),
        node.typeRestriction ? deepClone(*node.typeRestriction, alloc) : nullptr,
        *deepClone(node.declarators, alloc)
    );
}

SyntaxNode* clone(const TypeReferenceSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TypeReferenceSyntax>(
        node.typeKeyword.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const TypedefDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<TypedefDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.typedefKeyword.deepClone(alloc),
        *deepClone<DataTypeSyntax>(*node.type, alloc),
        node.name.deepClone(alloc),
        *deepClone(node.dimensions, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const UdpBodySyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UdpBodySyntax>(
        *deepClone(node.portDecls, alloc),
        node.initialStmt ? deepClone(*node.initialStmt, alloc) : nullptr,
        node.table.deepClone(alloc),
        *deepClone(node.entries, alloc),
        node.endtable.deepClone(alloc)
    );
}

SyntaxNode* clone(const UdpDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UdpDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.primitive.deepClone(alloc),
        node.name.deepClone(alloc),
        *deepClone<UdpPortListSyntax>(*node.portList, alloc),
        *deepClone<UdpBodySyntax>(*node.body, alloc),
        node.endprimitive.deepClone(alloc),
        node.endBlockName ? deepClone(*node.endBlockName, alloc) : nullptr
    );
}

SyntaxNode* clone(const UdpEdgeFieldSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UdpEdgeFieldSyntax>(
        node.openParen.deepClone(alloc),
        node.first.deepClone(alloc),
        node.second.deepClone(alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const UdpEntrySyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UdpEntrySyntax>(
        *deepClone(node.inputs, alloc),
        node.colon1.deepClone(alloc),
        node.current ? deepClone(*node.current, alloc) : nullptr,
        node.colon2.deepClone(alloc),
        node.next ? deepClone(*node.next, alloc) : nullptr,
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const UdpInitialStmtSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UdpInitialStmtSyntax>(
        node.initial.deepClone(alloc),
        node.name.deepClone(alloc),
        node.equals.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.value, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const UdpInputPortDeclSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UdpInputPortDeclSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        *deepClone(node.names, alloc)
    );
}

SyntaxNode* clone(const UdpOutputPortDeclSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UdpOutputPortDeclSyntax>(
        *deepClone(node.attributes, alloc),
        node.keyword.deepClone(alloc),
        node.reg.deepClone(alloc),
        node.name.deepClone(alloc),
        node.initializer ? deepClone(*node.initializer, alloc) : nullptr
    );
}

SyntaxNode* clone(const UdpSimpleFieldSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UdpSimpleFieldSyntax>(
        node.field.deepClone(alloc)
    );
}

SyntaxNode* clone(const UnaryBinsSelectExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UnaryBinsSelectExprSyntax>(
        node.op.deepClone(alloc),
        *deepClone<BinsSelectConditionExprSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const UnaryConditionalDirectiveExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UnaryConditionalDirectiveExpressionSyntax>(
        node.op.deepClone(alloc),
        *deepClone<ConditionalDirectiveExpressionSyntax>(*node.operand, alloc)
    );
}

SyntaxNode* clone(const UnaryPropertyExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UnaryPropertyExprSyntax>(
        node.op.deepClone(alloc),
        *deepClone<PropertyExprSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const UnarySelectPropertyExprSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UnarySelectPropertyExprSyntax>(
        node.op.deepClone(alloc),
        node.openBracket.deepClone(alloc),
        node.selector ? deepClone(*node.selector, alloc) : nullptr,
        node.closeBracket.deepClone(alloc),
        *deepClone<PropertyExprSyntax>(*node.expr, alloc)
    );
}

SyntaxNode* clone(const UnconditionalBranchDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UnconditionalBranchDirectiveSyntax>(
        node.kind,
        node.directive.deepClone(alloc),
        *deepClone(node.disabledTokens, alloc)
    );
}

SyntaxNode* clone(const UnconnectedDriveDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UnconnectedDriveDirectiveSyntax>(
        node.directive.deepClone(alloc),
        node.strength.deepClone(alloc)
    );
}

SyntaxNode* clone(const UndefDirectiveSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UndefDirectiveSyntax>(
        node.directive.deepClone(alloc),
        node.name.deepClone(alloc)
    );
}

SyntaxNode* clone(const UniquenessConstraintSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UniquenessConstraintSyntax>(
        node.unique.deepClone(alloc),
        *deepClone<RangeListSyntax>(*node.ranges, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const UserDefinedNetDeclarationSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<UserDefinedNetDeclarationSyntax>(
        *deepClone(node.attributes, alloc),
        node.netType.deepClone(alloc),
        *deepClone<TimingControlSyntax>(*node.delay, alloc),
        *deepClone(node.declarators, alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const ValueRangeExpressionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<ValueRangeExpressionSyntax>(
        node.openBracket.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.left, alloc),
        node.op.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.right, alloc),
        node.closeBracket.deepClone(alloc)
    );
}

SyntaxNode* clone(const VariableDimensionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<VariableDimensionSyntax>(
        node.openBracket.deepClone(alloc),
        node.specifier ? deepClone(*node.specifier, alloc) : nullptr,
        node.closeBracket.deepClone(alloc)
    );
}

SyntaxNode* clone(const VariablePatternSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<VariablePatternSyntax>(
        node.dot.deepClone(alloc),
        node.variableName.deepClone(alloc)
    );
}

SyntaxNode* clone(const VariablePortHeaderSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<VariablePortHeaderSyntax>(
        node.constKeyword.deepClone(alloc),
        node.direction.deepClone(alloc),
        node.varKeyword.deepClone(alloc),
        *deepClone<DataTypeSyntax>(*node.dataType, alloc)
    );
}

SyntaxNode* clone(const VirtualInterfaceTypeSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<VirtualInterfaceTypeSyntax>(
        node.virtualKeyword.deepClone(alloc),
        node.interfaceKeyword.deepClone(alloc),
        node.name.deepClone(alloc),
        node.parameters ? deepClone(*node.parameters, alloc) : nullptr,
        node.modport ? deepClone(*node.modport, alloc) : nullptr
    );
}

SyntaxNode* clone(const VoidCastedCallStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<VoidCastedCallStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.voidKeyword.deepClone(alloc),
        node.apostrophe.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const WaitForkStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<WaitForkStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.wait.deepClone(alloc),
        node.fork.deepClone(alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const WaitOrderStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<WaitOrderStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.wait_order.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone(node.names, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone<ActionBlockSyntax>(*node.action, alloc)
    );
}

SyntaxNode* clone(const WaitStatementSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<WaitStatementSyntax>(
        node.label ? deepClone(*node.label, alloc) : nullptr,
        *deepClone(node.attributes, alloc),
        node.wait.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc),
        *deepClone<StatementSyntax>(*node.statement, alloc)
    );
}

SyntaxNode* clone(const WildcardDimensionSpecifierSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<WildcardDimensionSpecifierSyntax>(
        node.star.deepClone(alloc)
    );
}

SyntaxNode* clone(const WildcardPatternSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<WildcardPatternSyntax>(
        node.dot.deepClone(alloc),
        node.star.deepClone(alloc)
    );
}

SyntaxNode* clone(const WildcardPortConnectionSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<WildcardPortConnectionSyntax>(
        *deepClone(node.attributes, alloc),
        node.dot.deepClone(alloc),
        node.star.deepClone(alloc)
    );
}

SyntaxNode* clone(const WildcardPortListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<WildcardPortListSyntax>(
        node.openParen.deepClone(alloc),
        node.dot.deepClone(alloc),
        node.star.deepClone(alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const WildcardUdpPortListSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<WildcardUdpPortListSyntax>(
        node.openParen.deepClone(alloc),
        node.dot.deepClone(alloc),
        node.star.deepClone(alloc),
        node.closeParen.deepClone(alloc),
        node.semi.deepClone(alloc)
    );
}

SyntaxNode* clone(const WithClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<WithClauseSyntax>(
        node.with.deepClone(alloc),
        node.openParen.deepClone(alloc),
        *deepClone<ExpressionSyntax>(*node.expr, alloc),
        node.closeParen.deepClone(alloc)
    );
}

SyntaxNode* clone(const WithFunctionClauseSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<WithFunctionClauseSyntax>(
        node.with.deepClone(alloc),
        *deepClone<NameSyntax>(*node.name, alloc)
    );
}

SyntaxNode* clone(const WithFunctionSampleSyntax& node, BumpAllocator& alloc) {
    return alloc.emplace<WithFunctionSampleSyntax>(
        node.with.deepClone(alloc),
        node.function.deepClone(alloc),
        node.sample.deepClone(alloc),
        node.portList ? deepClone(*node.portList, alloc) : nullptr
    );
}

}

namespace slang::syntax {

struct CloneVisitor {
    template<typename T>
    SyntaxNode* visit(const T& node, BumpAllocator& alloc) {
        if constexpr (requires { node.clone(alloc); }) {
            return node.clone(alloc);
        } else {
            return shallow::clone(node, alloc);
        }
    }
};

struct DeepCloneVisitor {
    template<typename T>
    SyntaxNode* visit(const T& node, BumpAllocator& alloc) {
        return deep::clone(node, alloc);
    }
};

SyntaxNode* deepClone(const SyntaxNode& node, BumpAllocator& alloc) {
    DeepCloneVisitor visitor;
    return node.visit(visitor, alloc);
}

SyntaxNode* clone(const SyntaxNode& node, BumpAllocator& alloc) {
    CloneVisitor visitor;
    return node.visit(visitor, alloc);
}

}
